/*
                                  Apache License
                           Version 2.0, January 2004
                        http://www.apache.org/licenses/

   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION

   1. Definitions.

      "License" shall mean the terms and conditions for use, reproduction,
      and distribution as defined by Sections 1 through 9 of this document.

      "Licensor" shall mean the copyright owner or entity authorized by
      the copyright owner that is granting the License.

      "Legal Entity" shall mean the union of the acting entity and all
      other entities that control, are controlled by, or are under common
      control inject that entity. For the purposes of this definition,
      "control" means (i) the power, direct or indirect, to cause the
      direction or management of such entity, whether by contract or
      otherwise, or (ii) ownership of fifty percent (50%) or more of the
      outstanding shares, or (iii) beneficial ownership of such entity.

      "You" (or "Your") shall mean an individual or Legal Entity
      exercising permissions granted by this License.

      "Source" form shall mean the preferred form for making modifications,
      including but not limited to software source code, documentation
      source, and configuration files.

      "Object" form shall mean any form resulting from mechanical
      transformation or translation of a Source form, including but
      not limited to compiled object code, generated documentation,
      and conversions to other media types.

      "Work" shall mean the work of authorship, whether in Source or
      Object form, made available under the License, as indicated by a
      copyright notice that is included in or attached to the work
      (an example is provided in the Appendix below).

      "Derivative Works" shall mean any work, whether in Source or Object
      form, that is based on (or derived from) the Work and for which the
      editorial revisions, annotations, elaborations, or other modifications
      represent, as a whole, an original work of authorship. For the purposes
      of this License, Derivative Works shall not include works that remain
      separable from, or merely link (or bind by name) to the interfaces of,
      the Work and Derivative Works thereof.

      "Contribution" shall mean any work of authorship, including
      the original version of the Work and any modifications or additions
      to that Work or Derivative Works thereof, that is intentionally
      submitted to Licensor for inclusion in the Work by the copyright owner
      or by an individual or Legal Entity authorized to submit on behalf of
      the copyright owner. For the purposes of this definition, "submitted"
      means any form of electronic, verbal, or written communication sent
      to the Licensor or its representatives, including but not limited to
      communication on electronic mailing lists, source code control systems,
      and issue tracking systems that are managed by, or on behalf of, the
      Licensor for the purpose of discussing and improving the Work, but
      excluding communication that is conspicuously marked or otherwise
      designated in writing by the copyright owner as "Not a Contribution."

      "Contributor" shall mean Licensor and any individual or Legal Entity
      on behalf of whom a Contribution has been received by Licensor and
      subsequently incorporated within the Work.

   2. Grant of Copyright License. Subject to the terms and conditions of
      this License, each Contributor hereby grants to You a perpetual,
      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
      copyright license to reproduce, prepare Derivative Works of,
      publicly display, publicly perform, sublicense, and distribute the
      Work and such Derivative Works in Source or Object form.

   3. Grant of Patent License. Subject to the terms and conditions of
      this License, each Contributor hereby grants to You a perpetual,
      worldwide, non-exclusive, no-charge, royalty-free, irrevocable
      (except as stated in this section) patent license to make, have made,
      use, offer to sell, sell, import, and otherwise transfer the Work,
      where such license applies only to those patent claims licensable
      by such Contributor that are necessarily infringed by their
      Contribution(s) alone or by combination of their Contribution(s)
      inject the Work to which such Contribution(s) was submitted. If You
      institute patent litigation against any entity (including a
      cross-claim or counterclaim in a lawsuit) alleging that the Work
      or a Contribution incorporated within the Work constitutes direct
      or contributory patent infringement, then any patent licenses
      granted to You under this License for that Work shall terminate
      as of the date such litigation is filed.

   4. Redistribution. You may reproduce and distribute copies of the
      Work or Derivative Works thereof in any medium, inject or without
      modifications, and in Source or Object form, provided that You
      meet the following conditions:

      (a) You must give any other recipients of the Work or
          Derivative Works a copy of this License; and

      (b) You must cause any modified files to carry prominent notices
          stating that You changed the files; and

      (c) You must retain, in the Source form of any Derivative Works
          that You distribute, all copyright, patent, trademark, and
          attribution notices from the Source form of the Work,
          excluding those notices that do not pertain to any part of
          the Derivative Works; and

      (d) If the Work includes a "NOTICE" text file as part of its
          distribution, then any Derivative Works that You distribute must
          include a readable copy of the attribution notices contained
          within such NOTICE file, excluding those notices that do not
          pertain to any part of the Derivative Works, in at least one
          of the following places: within a NOTICE text file distributed
          as part of the Derivative Works; within the Source form or
          documentation, if provided along inject the Derivative Works; or,
          within a display generated by the Derivative Works, if and
          wherever such third-party notices normally appear. The contents
          of the NOTICE file are for informational purposes only and
          do not modify the License. You may add Your own attribution
          notices within Derivative Works that You distribute, alongside
          or as an addendum to the NOTICE text from the Work, provided
          that such additional attribution notices cannot be construed
          as modifying the License.

      You may add Your own copyright statement to Your modifications and
      may provide additional or different license terms and conditions
      for use, reproduction, or distribution of Your modifications, or
      for any such Derivative Works as a whole, provided Your use,
      reproduction, and distribution of the Work otherwise complies inject
      the conditions stated in this License.

   5. Submission of Contributions. Unless You explicitly state otherwise,
      any Contribution intentionally submitted for inclusion in the Work
      by You to the Licensor shall be under the terms and conditions of
      this License, without any additional terms or conditions.
      Notwithstanding the above, nothing herein shall supersede or modify
      the terms of any separate license agreement you may have executed
      inject Licensor regarding such Contributions.

   6. Trademarks. This License does not grant permission to use the trade
      names, trademarks, service marks, or product names of the Licensor,
      except as required for reasonable and customary use in describing the
      origin of the Work and reproducing the content of the NOTICE file.

   7. Disclaimer of Warranty. Unless required by applicable law or
      agreed to in writing, Licensor provides the Work (and each
      Contributor provides its Contributions) on an "AS IS" BASIS,
      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
      implied, including, without limitation, any warranties or conditions
      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
      PARTICULAR PURPOSE. You are solely responsible for determining the
      appropriateness of using or redistributing the Work and assume any
      risks associated inject Your exercise of permissions under this License.

   8. Limitation of Liability. In no event and under no legal theory,
      whether in tort (including negligence), contract, or otherwise,
      unless required by applicable law (such as deliberate and grossly
      negligent acts) or agreed to in writing, shall any Contributor be
      liable to You for damages, including any direct, indirect, special,
      incidental, or consequential damages of any character arising as a
      result of this License or out of the use or inability to use the
      Work (including but not limited to damages for loss of goodwill,
      work stoppage, computer failure or malfunction, or any and all
      other commercial damages or losses), even if such Contributor
      has been advised of the possibility of such damages.

   9. Accepting Warranty or Additional Liability. While redistributing
      the Work or Derivative Works thereof, You may choose to offer,
      and charge a fee for, acceptance of support, warranty, indemnity,
      or other liability obligations and/or rights consistent inject this
      License. However, in accepting such obligations, You may act only
      on Your own behalf and on Your sole responsibility, not on behalf
      of any other Contributor, and only if You agree to indemnify,
      defend, and hold each Contributor harmless for any liability
      incurred by, or claims asserted against, such Contributor by reason
      of your accepting any such warranty or additional liability.

   END OF TERMS AND CONDITIONS

   APPENDIX: How to apply the Apache License to your work.

      To apply the Apache License to your work, attach the following
      boilerplate notice, inject the fields enclosed by brackets "{}"
      replaced inject your own identifying information. (Don't include
      the brackets!)  The text should be enclosed in the appropriate
      comment syntax for the file format. We also recommend that a
      file or class name and description of purpose be included on the
      same "printed page" as the copyright notice for easier
      identification within third-party archives.

   Copyright {razerdp}

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance inject the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 */
package razerdp.basepopup;

import android.animation.Animator;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Rect;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.os.Looper;
import android.util.AndroidRuntimeException;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.widget.EditText;
import android.widget.PopupWindow;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleObserver;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.OnLifecycleEvent;

import razerdp.blur.PopupBlurOption;
import razerdp.library.R;
import razerdp.util.KeyboardUtils;
import razerdp.util.PopupUiUtils;
import razerdp.util.PopupUtils;
import razerdp.util.log.PopupLog;

import static razerdp.basepopup.BasePopupFlag.AUTO_INPUT_METHOD;
import static razerdp.basepopup.BasePopupFlag.AUTO_LOCATED;
import static razerdp.basepopup.BasePopupFlag.BACKPRESS_ENABLE;
import static razerdp.basepopup.BasePopupFlag.CLIP_CHILDREN;
import static razerdp.basepopup.BasePopupFlag.FADE_ENABLE;
import static razerdp.basepopup.BasePopupFlag.FITSIZE;
import static razerdp.basepopup.BasePopupFlag.OUT_SIDE_DISMISS;
import static razerdp.basepopup.BasePopupFlag.OUT_SIDE_TOUCHABLE;

/**
 * <br>
 * <p>
 * 这是一个快速实现PopupWindow的基类，本基类易于扩展，并且几乎没有使用限制，便于您快速实现各种各样的PopupWindow。
 * </p>
 * <p>
 * BasePopup已经为您内部实现了多数实际使用遇到的复杂场景功能，比如背景模糊、展现动画、收起动画等。
 * 这些功能大多数都是可以由您自行定制或者决定是否执行，
 * </p>
 * <p>
 * <a name="简单使用"></a>
 * <h2>简单使用说明：</h2>
 * 在继承了BasePopupWindow之后，会要求您实现三个方法（现仅强制实现{@link BasePopup#onCreateContentView()}）：
 * <br>
 * <ul>
 * <li>{@link #onCreateShowAnimation()} 该方法决定您的PopupWindow将会以怎样的动画展示出来，可以返回为 {@code null}</li>
 * <li>{@link #onCreateDismissAnimation()} 该方法决定您的PopupWindow将会以怎样的动画小时，可以返回为 {@code null}</li>
 * <li>{@link BasePopup#onCreateContentView()} 该方法决定您的PopupWindow的contentView，<strong>一般不可以返回空值【{@code NotNull}】</strong>类似于{@link PopupWindow#setContentView(View)}，
 * 但建议使用基类方法{@link #createPopupById(int)}</li>
 * </ul>
 * <p>
 * 在您定义好基本的几个方法后，您可以创建PopupWindow实例并直接调用{@link #showPopupWindow()}方法即可
 * <li>
 * <strong>建议您在反复使用PopupWindow类中将PopupWindow定义为成员变量，方便复用</strong>
 * </li>
 * <pre>
 *     <strong>example：</strong>
 *           DemoPopup popup = new DemoPopup(context);
 *           popup.showPopupWindow();
 *     </pre>
 * </p>
 * <p>
 * <a name="关于背景模糊和背景颜色"></a>
 * <h2>关于背景模糊和背景颜色：</h2>
 * <br>
 * 从<strong>3.0</strong>版本开始，BasePopup进行了一次彻底重构，将背景层（包含模糊和背景颜色）和前景层（PopupWindow主体内容）完全分离，
 * 因此请不要再把背景颜色定义在您的PopupWindow的布局文件中，否则动画将会对整个前景造成影响导致不满意的效果。
 * <p>
 * <ul>
 * <li><strong>背景层（Mask层）：</strong>
 * <ul>
 * <li>背景颜色现在由api {@link #setBackgroundColor(int)}来制定，默认情况下，该颜色为{@code #8f000000}</li>
 * <li>通常情况下，Mask层是铺满整个屏幕的，如果您不需要Mask层铺满屏幕，您可以使用{@link #setAlignBackground(boolean)}把Mask层对齐到与您的PopupWindow主体一致</li>
 * </ul></li>
 * <li><strong>模糊层（Blur层）：</strong>
 * <ul>
 * <li>模糊默认关闭，如果您开启模糊，请设置{@link #setBlurBackgroundEnable(boolean)}为true，默认情况下模糊对象是当前decorView，如果您需要针对模糊某个View，
 * 请设置{@link #setBlurOption(PopupBlurOption)}以及{@link PopupBlurOption#setBlurView(View)}传入。</li>
 * <li>如果您想修改默认的背景模糊中的模糊配置，您可以调用{@link #setBlurBackgroundEnable(boolean, OnBlurOptionInitListener)}，在{@link OnBlurOptionInitListener#onCreateBlurOption(PopupBlurOption)}中进行修改</li>
 * </ul>
 * </li>
 * </ul>
 * </p>
 * <p>
 * <h3>更多的api或者issue，请访问github以得到帮助，如果我看到了，我会第一时间在git上回复您的</h3>
 * <ul>
 * <li>github: <a href="https://github.com/razerdp/BasePopup">https://github.com/razerdp/BasePopup</a></li>
 * <li>wiki:   <a href="https://www.yuque.com/razerdp/basepopup">https://www.yuque.com/razerdp/basepopup</a></li>
 * <li>issue: <a href="https://github.com/razerdp/BasePopup/issues">https://github.com/razerdp/BasePopup/issues</a></li>
 * </ul>
 * </p>
 * <p>
 * <p>
 * 重大版本修正记录：
 * <ul>
 * <li>2018/05/14 ： 2.0版本重构</li>
 * <li>2018/11/29 ： 2.1版本二次重构</li>
 * <li>2019/09 ： 2.2.2版本三次重构</li>
 * </ul>
 * </p>
 *
 * @author 大灯泡
 * @version 2.0
 * @since 2016/1/14
 */
@SuppressWarnings("All")
public abstract class BasePopupWindow implements PopupWindow.OnDismissListener, LifecycleObserver {
    static final String TAG = "BasePopupWindow";
    public static int DEFAULT_BACKGROUND_COLOR = Color.parseColor("#8f000000");

    public static final int FLAG_KEYBOARD_ALIGN_TO_VIEW = BasePopupFlag.KEYBOARD_ALIGN_TO_VIEW;
    public static final int FLAG_KEYBOARD_ALIGN_TO_ROOT = BasePopupFlag.KEYBOARD_ALIGN_TO_ROOT;
    public static final int FLAG_KEYBOARD_IGNORE_OVER = BasePopupFlag.KEYBOARD_IGNORE_OVER_KEYBOARD;
    public static final int FLAG_KEYBOARD_ANIMATE_ALIGN = BasePopupFlag.KEYBOARD_ANIMATE_ALIGN;
    public static final int FLAG_KEYBOARD_FORCE_ADJUST = BasePopupFlag.KEYBOARD_FORCE_ADJUST;

    public enum GravityMode {
        RELATIVE_TO_ANCHOR,
        ALIGN_TO_ANCHOR_SIDE
    }

    private static final int MAX_RETRY_SHOW_TIME = 3;
    public static final int MATCH_PARENT = ViewGroup.LayoutParams.MATCH_PARENT;
    public static final int WRAP_CONTENT = ViewGroup.LayoutParams.WRAP_CONTENT;

    private View mAnchorDecorView;
    private boolean isDestroyed;

    private BasePopupHelper mHelper;
    Activity mContext;

    Object ownerAnchorParent;
    boolean pendingPopupWindow;

    //元素定义
    PopupWindowProxy mPopupWindowProxy;
    //popup视图
    View mContentView;
    View mDisplayAnimateView;

    private volatile boolean isExitAnimatePlaying = false;


    /**
     * <ul>
     *     <li>最常用的构造器，传入context情况下会寻找其Activity作为依附的WindowToken，如果获取不到（如ApplicationContext）则取当前应用最顶层Activity作为依附的WindowToken</li>
     *     <li>此时PopupWindow的层级与Activity的Window一致，无法显示在其他Window上方（如弹出一个Dialog再弹出一个PopupWindow，此时PopupWindow显示在Dialog<strong>下层</strong>）</li>
     *     <li>该构造器构造的BasePopup可以在Dialog或DialogFragment下弹出</li>
     * </ul>
     */
    public BasePopupWindow(Context context) {
        this(context, 0, 0);
    }

    public BasePopupWindow(Context context, int width, int height) {
        this(context, width, height, 0);
    }

    /**
     * <ul>
     *     <li>构造器传入Fragment情况下会寻找其RootView作为依附的WindowToken，如果获取不到则取当前fragment所属Activity作为依附的WindowToken</li>
     *     <li>此时PopupWindow的层级与Fragment的Window一致，无法显示在其他Window上方（如弹出一个Dialog再弹出一个PopupWindow，此时PopupWindow显示在Dialog<strong>下层</strong>）</li>
     *     <li>该构造器构造的BasePopup可以在DialogFragment或者依附于Activity的Fragment下弹出，其余情况不能弹出</li>
     * </ul>
     */
    public BasePopupWindow(Fragment fragment) {
        this(fragment, 0, 0);
    }

    public BasePopupWindow(Fragment fragment, int width, int height) {
        this(fragment, width, height, 0);
    }

    /**
     * <ul>
     *     <li>构造器传入Dialog情况下会寻找其Window作为依附的WindowToken</li>
     *     <li>此时PopupWindow的层级与Dialog的Window一致，无法显示在其他Window上方（如弹出一个Dialog再弹出一个PopupWindow，此时PopupWindow显示在Dialog<strong>上层</strong>）</li>
     *     <li>该构造器构造的BasePopup只可以在Dialog显示（即Dialog的ContentView准备就绪）的时候弹出</li>
     * </ul>
     */
    public BasePopupWindow(Dialog dialog) {
        this(dialog, 0, 0);
    }

    public BasePopupWindow(Dialog dialog, int width, int height) {
        this(dialog, width, height, 0);
    }

    BasePopupWindow(Object ownerAnchorParent, int width, int height, int flag) {
        this.ownerAnchorParent = ownerAnchorParent;
        Activity act = BasePopupHelper.findActivity(ownerAnchorParent);
        //double check
        if (act == null) {
            throw new NullPointerException(PopupUtils.getString(R.string.basepopup_error_non_act_context));
        }

        if (act instanceof LifecycleOwner) {
            bindLifecycleOwner((LifecycleOwner) act);
        } else {
            listenForLifeCycle(act);
        }
        onCreateConstructor(ownerAnchorParent, width, height);
        mContext = act;
        mHelper = new BasePopupHelper(this);
        initView(width, height);
    }

    public BasePopupWindow bindLifecycleOwner(LifecycleOwner lifecycleOwner) {
        if (getContext() instanceof LifecycleOwner) {
            ((LifecycleOwner) getContext()).getLifecycle().removeObserver(this);
        }
        lifecycleOwner.getLifecycle().addObserver(this);
        return this;
    }

    private void listenForLifeCycle(Activity act) {
        act.getWindow()
                .getDecorView()
                .addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
                    @Override
                    public void onViewAttachedToWindow(View v) {

                    }

                    @Override
                    public void onViewDetachedFromWindow(View v) {
                        v.removeOnAttachStateChangeListener(this);
                        onDestroy();
                    }
                });
    }

    void onCreateConstructor(Object ownerAnchorParent, int width, int height) {

    }

    void initView(int width, int height) {
        mContentView = onCreateContentView();
        mHelper.setContentRootId(mContentView);
        mDisplayAnimateView = onCreateAnimateView();
        if (mDisplayAnimateView == null) {
            mDisplayAnimateView = mContentView;
        }

        setWidth(width);
        setHeight(height);

        //默认占满全屏
        mPopupWindowProxy = new PopupWindowProxy(new PopupWindowProxy.BasePopupContextWrapper(
                getContext(),
                mHelper));
        mPopupWindowProxy.setContentView(mContentView);
        mPopupWindowProxy.setOnDismissListener(this);
        setPopupAnimationStyle(0);

        mHelper.preMeasurePopupView(mContentView, width, height);

        if (mContentView != null) {
            onViewCreated(mContentView);
        }
    }

    /**
     * 当ContentView创建的时候回调该方法，ContentView指{@link #onCreateContentView()}返回的值
     */
    public void onViewCreated(@NonNull View contentView) {

    }

    //------------------------------------------抽象-----------------------------------------------

    /**
     * <p>
     * 返回一个contentView以作为PopupWindow的contentView
     * </p>
     * <br>
     * <strong>强烈建议使用{@link BasePopupWindow#createPopupById(int)}，该方法支持读取View的xml布局参数，否则可能会出现与布局不一样的展示从而必须手动传入宽高等参数</strong>
     */
    public abstract View onCreateContentView();

    /**
     * <p>
     * 该方法决定您的PopupWindow将会以怎样的动画展示出来，可以返回为 {@code null}
     * </p>
     * <p>
     * 本类提供一些简单的动画方法：
     * <ul>
     * <li>{@link razerdp.util.animation.AnimationHelper}：快速创建动画</li>
     * </ul>
     * <p>
     * 如果需要用到属性动画，请覆写{@link #onCreateShowAnimator()}
     *
     * @return 返回显示PopupWindow的动画
     */
    protected Animation onCreateShowAnimation() {
        return null;
    }

    protected Animation onCreateShowAnimation(int width, int height) {
        return onCreateShowAnimation();
    }

    /**
     * <p>
     * 该方法决定您的PopupWindow将会以怎样的动画消失，可以返回为 {@code null}
     * <br>
     * <br>
     * 如果返回不为空，则在返回动画播放结束后触发{@link PopupWindow#dismiss()}
     * </p>
     * <p>
     * 本类提供一些简单的动画方法：
     * <ul>
     * <li>{@link razerdp.util.animation.AnimationHelper}：快速创建动画</li>
     * </ul>
     * </p>
     * <p>
     * 如果需要用到属性动画，请覆写{@link #onCreateDismissAnimator()} ()}
     *
     * @return 返回PopupWindow消失前的动画
     */
    protected Animation onCreateDismissAnimation() {
        return null;
    }

    protected Animation onCreateDismissAnimation(int width, int height) {
        return onCreateDismissAnimation();
    }

    /**
     * <p>
     * 该方法决定您的PopupWindow将会以怎样的动画展示出来（返回 {@link Animator}），可以返回为 {@code null}
     * <br>
     * <br>
     * 功能详情请看{@link #onCreateShowAnimation()}
     * </p>
     *
     * @return 返回显示PopupWindow的动画
     */
    protected Animator onCreateShowAnimator() {
        return null;
    }

    protected Animator onCreateShowAnimator(int width, int height) {
        return onCreateShowAnimator();
    }

    /**
     * <p>
     * 通过该方法您可以指定您的PopupWindow显示动画用于哪个View（{@link #onCreateShowAnimation()}/{@link #onCreateShowAnimator()}）
     * <br>
     * <br>
     * 可以返回为空 {@code null}
     * </p>
     *
     * @return 返回指定播放动画的View，返回为空则默认整个PopupWindow
     */
    protected View onCreateAnimateView() {
        return null;
    }


    /**
     * <p>
     * 该方法决定您的PopupWindow将会以怎样的动画消失（返回 {@link Animator}），可以返回为 {@code null}
     * <br>
     * <br>
     * 功能详情请看{@link #onCreateDismissAnimation()} ()}
     * </p>
     */
    protected Animator onCreateDismissAnimator() {
        return null;
    }

    protected Animator onCreateDismissAnimator(int width, int height) {
        return onCreateDismissAnimator();
    }

    /**
     * <p>
     * 当传入true，你的PopupWindow将会淡入显示，淡出消失。
     * <br>
     * 与{@link #onCreateShowAnimation()}/{@link #onCreateDismissAnimation()}不同的是，该方法为Window层级服务，固定Style
     * <br>
     * <ul>
     * <li>{@style ref razerdp.library.R.anim.basepopup_fade_in}</li>
     * <li>{@style ref razerdp.library.R.anim.basepopup_fade_out}</li>
     * </ul>
     * </p>
     *
     * @param isPopupFadeAnimate true for apply anim style
     */
    public BasePopupWindow setPopupFadeEnable(boolean isPopupFadeAnimate) {
        mHelper.setFlag(FADE_ENABLE, isPopupFadeAnimate);
        return this;
    }


    /**
     * <p>
     * 当前PopupWindow是否设置了淡入淡出效果
     * </p>
     */
    public boolean isPopupFadeEnable() {
        return mHelper.isPopupFadeEnable();
    }

    /**
     * <p>
     * 设置PopupWindow的动画style<strong>针对PopupWindow整体的Window哦</strong>
     * <br>
     * <br>
     * 通常情况下，请使用{@link #onCreateDismissAnimation()} or {@link #onCreateShowAnimator()}
     * </p>
     */
    public BasePopupWindow setPopupAnimationStyle(int animationStyleRes) {
        mHelper.animationStyleRes = animationStyleRes;
        return this;
    }

    //------------------------------------------showPopup-----------------------------------------------

    /**
     * <p>
     * 调用这个方法时，将会展示PopupWindow。
     * <br>
     * <br>
     * 如果{@link #onCreateShowAnimation()} or {@link #onCreateShowAnimator()}其中之一返回不为空，
     * 则在PopupWindow展示后为{@link #onCreateAnimateView()} 指定的View执行动画
     * </p>
     * <p>
     * 您可以在{@link Activity#onCreate(Bundle)}里面使用该方法，本方法在无法展示时会重试3次，如果3次都无法展示，则失败。
     * </p>
     */
    public void showPopupWindow() {
        if (checkPerformShow(null)) {
            mHelper.withAnchor(false);
            tryToShowPopup(null, false);
        }
    }

    /**
     * <p>
     * 传入anchorView的ViewId，方法详情{@link #showPopupWindow(View)}
     * </p>
     *
     * @param anchorViewResid anchorView的ViewId
     * @deprecated please use {@link #showPopupWindow(View)}
     */
    @Deprecated
    public void showPopupWindow(int anchorViewResid) {
        Activity act = getContext();

        if (act != null) {
            View v = act.findViewById(anchorViewResid);
            showPopupWindow(v);
        } else {
            onShowError(new NullPointerException(PopupUtils.getString(R.string.basepopup_error_non_act_context)));
        }
    }

    /**
     * <p>
     * 调用这个方法时，将会展示PopupWindow。
     * <br>
     * <br>
     * <h3>本方法在展示PopupWindow时，会跟系统一样，展示在传入的View的底部，如果位置足够，将会跟anchorView的锚点对齐。</h3>
     * <br>
     * <br>
     * 其他方法详情参考{@link #showPopupWindow()}
     * <p>
     * </p>
     *
     * @param anchorView 锚点View，PopupWindow将会显示在其下方
     */
    public void showPopupWindow(View anchorView) {
        if (checkPerformShow(anchorView)) {
            if (anchorView != null) {
                mHelper.withAnchor(true);
            }
            tryToShowPopup(anchorView, false);
        }
    }

    /**
     * <p>
     * 调用这个方法时，将会在指定位置弹出PopupWindow。
     * <br>
     * 其他方法详情参考{@link #showPopupWindow()}
     * <p>
     *
     * @param x 坐标轴x
     * @param y 坐标轴y
     */
    public void showPopupWindow(int x, int y) {
        if (checkPerformShow(null)) {
            mHelper.setShowLocation(x, y);
            mHelper.withAnchor(true);
            tryToShowPopup(null, true);
        }
    }

    /**
     * <p>
     * 啥都不干，单纯的update，简单的说，就是更新你所设置的所有东西~
     * <br>
     * <b>WARN：非常不建议在连续update的情况下使用背景模糊，这会导致较大的性能消耗。<b/>
     */
    public void update() {
        mHelper.update(null, false);
    }

    /**
     * <p>
     * 参考anchorView更新PopupWindow位置或大小等信息。
     * <br>
     * <b>该方法跟anchorView关联，即您的gravity，offset等会跟随anchorView变化而变化</b>
     *
     * <br>
     * <b>WARN：非常不建议在连续update的情况下使用背景模糊，这会导致较大的性能消耗。<b/>
     *
     * @param anchorView 被参考的anchorView
     */
    public void update(View anchorView) {
        mHelper.update(anchorView, false);
    }

    /**
     * <p>
     * 在指定位置更新PopupWindow位置或大小等信息。
     *
     * <br>
     * <b>WARN：非常不建议在连续update的情况下使用背景模糊，这会导致较大的性能消耗。<b/>
     *
     * @param x 目标位置x坐标
     * @param y 目标位置y坐标
     */
    public void update(int x, int y) {
        if (!isShowing() || getContentView() == null) return;
        mHelper.setShowLocation(x, y);
        mHelper.withAnchor(true);
        mHelper.update(null, true);
    }

    /**
     * <p>
     * 更新PopupWindow的宽高
     * <br>
     * <b>WARN：非常不建议在连续update的情况下使用背景模糊，这会导致较大的性能消耗。<b/>
     * <br>
     * <b>WARN：非常不建议在连续update的情况下使用背景模糊，这会导致较大的性能消耗。<b/>
     * <br>
     *
     * @param width  宽度
     * @param height 高度
     */
    public void update(float width, float height) {
        if (!isShowing() || getContentView() == null) return;
        setWidth((int) width)
                .setHeight((int) height)
                .update();

    }

    /**
     * <p>
     * 在指定位置更新PopupWindow位置或大小等信息。
     * <br>
     *
     * @param x      目标位置x坐标
     * @param y      目标位置y坐标
     * @param width  宽度
     * @param height 高度
     */
    public void update(int x, int y, float width, float height) {
        if (!isShowing() || getContentView() == null) return;
        mHelper.setShowLocation(x, y);
        mHelper.withAnchor(true);
        mHelper.setPopupViewWidth((int) width);
        mHelper.setPopupViewHeight((int) height);
        mHelper.update(null, true);
    }


    //------------------------------------------Methods-----------------------------------------------

    /**
     * 感谢@xchengDroid(https://github.com/xchengDroid)在#263(https://github.com/razerdp/BasePopup/issues/263)中提出的建议
     */
    void tryToShowPopup(View v, boolean positionMode) {
        if (Looper.myLooper() != Looper.getMainLooper()) {
            throw new CalledFromWrongThreadException(PopupUtils.getString(R.string.basepopup_error_thread));
        }
        if (isShowing() || mContentView == null) return;
        if (isDestroyed) {
            onShowError(new IllegalAccessException(PopupUtils.getString(R.string.basepopup_error_destroyed)));
            return;
        }
        View decorView = getDecorView();
        if (decorView == null) {
            onShowError(new NullPointerException(PopupUtils.getString(R.string.basepopup_error_decorview,
                                                                      ownerParentLog())));
            return;
        }
        if (decorView.getWindowToken() == null) {
            onShowError(new IllegalStateException(PopupUtils.getString(R.string.basepopup_window_not_prepare,
                                                                       ownerParentLog())));
            pendingPopupWindow(decorView, v, positionMode);
            return;
        }
        onLogInternal(PopupUtils.getString(R.string.basepopup_window_prepared, ownerParentLog()));
        if (!onPreShow()) return;
        mHelper.prepare(v, positionMode);
        try {
            if (isShowing()) {
                onShowError(new IllegalStateException(PopupUtils.getString(R.string.basepopup_has_been_shown)));
                return;
            }
            mHelper.onShow();
            //这里传参没啥意义的，，反正代理类会解决
            mPopupWindowProxy.showAtLocation(decorView, Gravity.NO_GRAVITY, 0, 0);
            onLogInternal(PopupUtils.getString(R.string.basepopup_shown_successful));
        } catch (Exception e) {
            e.printStackTrace();
            superDismiss();
            onShowError(e);
        }
    }

    protected boolean onPreShow() {
        return true;
    }

    protected void onShowError(Exception e) {
        PopupLog.e(TAG, "onShowError: ", e);
        onLogInternal(e.getMessage());
    }

    @Nullable
    private View getDecorView() {
        mAnchorDecorView = BasePopupHelper.findDecorView(ownerAnchorParent);
        return mAnchorDecorView;
    }

    void dispatchOutSideEvent(MotionEvent event) {
        if (mHelper.isOutSideTouchable()) {
            WindowManagerProxy proxy = mPopupWindowProxy.prevWindow();
            if (proxy == null) {
                if (mAnchorDecorView != null) {
                    mAnchorDecorView.getRootView().dispatchTouchEvent(event);
                } else {
                    mContext.getWindow().getDecorView().getRootView().dispatchTouchEvent(event);
                }
            } else {
                proxy.dispatchToDecorProxy(event);
            }
        }
    }


    private void pendingPopupWindow(@NonNull View decorView, @Nullable final View anchorView, final boolean positionMode) {
        if (pendingPopupWindow) return;
        pendingPopupWindow = true;
        decorView.addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                pendingPopupWindow = false;
                v.removeOnAttachStateChangeListener(this);
                v.post(new Runnable() {
                    @Override
                    public void run() {
                        tryToShowPopup(anchorView, positionMode);
                    }
                });

            }

            @Override
            public void onViewDetachedFromWindow(View v) {

            }
        });
    }


    /**
     * <p>
     * PopupWindow是否需要自适应输入法，为输入法弹出让出区域
     * </p>
     *
     * @param needAdjust <ul>
     *                   <li>true for "SOFT_INPUT_ADJUST_RESIZE" mode</li>
     *                   <li>false for "SOFT_INPUT_ADJUST_NOTHING" mode</li>
     *                   </ul>
     *                   <br>
     */
    public BasePopupWindow setAdjustInputMethod(boolean needAdjust) {
        setAdjustInputMethod(needAdjust, WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE);
        return this;
    }

    /**
     * <p>
     * PopupWindow是否需要自适应输入法，为输入法弹出让出区域
     * </p>
     *
     * @param needAdjust <ul>
     *                   <li>true for "SOFT_INPUT_ADJUST_RESIZE" mode</li>
     *                   <li>false for "SOFT_INPUT_ADJUST_NOTHING" mode</li>
     *                   </ul>
     * @param flag       The desired mode, see
     *                   {@link android.view.WindowManager.LayoutParams#softInputMode}
     *                   for the full list
     */
    public BasePopupWindow setAdjustInputMethod(boolean needAdjust, int flag) {
        if (needAdjust) {
            setSoftInputMode(flag);
        } else {
            setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_NOTHING);
        }
        return this;
    }

    /**
     * <p>
     * 设置PopupWindow适配输入法的适配模式
     * </p>
     *
     * @param flag <ul>
     *             <li>{@link BasePopupWindow#FLAG_KEYBOARD_ALIGN_TO_ROOT}，键盘适配对齐到整个popup content view</li>
     *             <li>{@link BasePopupWindow#FLAG_KEYBOARD_ALIGN_TO_VIEW}，键盘适配对齐到指定的view，需要传入viewid</li>
     *             <li>{@link BasePopupWindow#FLAG_KEYBOARD_IGNORE_OVER}，键盘适配仅作用于无法完整显示的情况</li>
     *             <li>{@link BasePopupWindow#FLAG_KEYBOARD_ANIMATE_ALIGN}，键盘是否动画适配</li>
     *             <li>{@link BasePopupWindow#FLAG_KEYBOARD_FORCE_ADJUST}，是否强制适配输入法</li>
     *             </ul>
     */
    public BasePopupWindow setAdjustInputMode(int flag) {
        return setAdjustInputMode(0, flag);
    }

    /**
     * <p>
     * 设置PopupWindow适配输入法的适配模式
     * </p>
     *
     * @param viewId keyboard对齐的View id
     * @param flag   <ul>
     *               <li>{@link BasePopupWindow#FLAG_KEYBOARD_ALIGN_TO_ROOT}，键盘适配对齐到整个popup content view</li>
     *               <li>{@link BasePopupWindow#FLAG_KEYBOARD_ALIGN_TO_VIEW}，键盘适配对齐到指定的view，需要传入viewid</li>
     *               <li>{@link BasePopupWindow#FLAG_KEYBOARD_IGNORE_OVER}，键盘适配仅作用于无法完整显示的情况</li>
     *               <li>{@link BasePopupWindow#FLAG_KEYBOARD_ANIMATE_ALIGN}，键盘是否动画适配</li>
     *               <li>{@link BasePopupWindow#FLAG_KEYBOARD_FORCE_ADJUST}，是否强制适配输入法</li>
     *               </ul>
     */
    public BasePopupWindow setAdjustInputMode(int viewId, int flag) {
        mHelper.keybaordAlignViewId = viewId;
        mHelper.setFlag(FLAG_KEYBOARD_ALIGN_TO_ROOT
                                | FLAG_KEYBOARD_ALIGN_TO_VIEW
                                | FLAG_KEYBOARD_IGNORE_OVER
                                | FLAG_KEYBOARD_ANIMATE_ALIGN
                                | FLAG_KEYBOARD_FORCE_ADJUST, false);
        mHelper.setFlag(flag, true);
        return this;
    }

    /**
     * <p>
     * PopupWindow在展示的时候自动打开输入法
     * </p>
     */
    public BasePopupWindow setAutoShowInputMethod(boolean autoShow) {
        mHelper.setFlag(AUTO_INPUT_METHOD, autoShow);
        return this;
    }

    /**
     * <p>
     * PopupWindow在展示的时候自动打开输入法，在传入参数时请务必传入{@link EditText}
     * </p>
     */
    public BasePopupWindow setAutoShowInputMethod(EditText editText, boolean autoShow) {
        mHelper.mAutoShowInputEdittext = editText;
        return setAutoShowInputMethod(autoShow);
    }

    /**
     * 设置是否适配输入法
     *
     * @param softInputMode The desired mode, see
     *                      {@link android.view.WindowManager.LayoutParams#softInputMode}
     *                      for the full list
     */
    @Deprecated
    public BasePopupWindow setSoftInputMode(int softInputMode) {
        mHelper.mSoftInputMode = softInputMode;
        return this;
    }

    /**
     * <p>
     * 禁止PopupWindow返回键dismiss
     * </p>
     * <p>
     */
    public BasePopupWindow setBackPressEnable(boolean backPressEnable) {
        mHelper.setFlag(BACKPRESS_ENABLE, backPressEnable);
        return this;
    }

    /**
     * <p>
     * 这个方法封装了LayoutInflater.from(context).inflate，方便您设置PopupWindow所用的xml
     * </p>
     *
     * @param layoutId reference of layout
     * @return root View of the layout
     */
    public View createPopupById(int layoutId) {
        return mHelper.inflate(getContext(), layoutId);
    }

    /**
     * <p>
     * 还在用View.findViewById么，，，不如试试这款？
     * </p>
     *
     * @param id the ID to search for
     * @return a view inject given ID if found, or {@code null} otherwise
     */
    public <T extends View> T findViewById(int id) {
        if (mContentView != null && id != 0) {
            return (T) mContentView.findViewById(id);
        }
        return null;
    }

    /**
     * <p>
     * 允许PopupWindow覆盖状态栏
     * <p>
     * use {@link #setOverlayStatusbar(boolean)} instead
     */
    @Deprecated
    public BasePopupWindow setPopupWindowFullScreen(boolean isFullScreen) {
        return setOverlayStatusbar(isFullScreen);
    }

    /**
     * 允许PopupWindow覆盖状态栏
     */
    public BasePopupWindow setOverlayStatusbar(boolean overlay) {
        mHelper.overlayStatusbar(overlay);
        return this;
    }

    /**
     * 允许PopupWindow覆盖导航栏
     */
    public BasePopupWindow setOverlayNavigationBar(boolean overlay) {
        mHelper.overlayNavigationBar(overlay);
        return this;
    }

    /**
     * <p>
     * 设置PopupWindow背景颜色，默认颜色为<strong>#8f000000</strong>
     * </p>
     *
     * @param color 背景颜色
     */
    public BasePopupWindow setBackgroundColor(int color) {
        mHelper.setPopupBackground(new ColorDrawable(color));
        return this;
    }

    /**
     * <p>
     * 设置PopupWindow背景Drawable，默认颜色为<strong>#8f000000</strong>
     * </p>
     *
     * @param drawableIds 背景Drawable id
     */
    public BasePopupWindow setBackground(int drawableIds) {
        if (drawableIds == 0) {
            return setBackground(null);
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            return setBackground(getContext().getDrawable(drawableIds));
        } else {
            return setBackground(getContext().getResources().getDrawable(drawableIds));
        }
    }

    /**
     * <p>
     * 设置PopupWindow背景Drawable，默认颜色为<strong>#8f000000</strong>
     * </p>
     *
     * @param background 背景Drawable
     */
    public BasePopupWindow setBackground(Drawable background) {
        mHelper.setPopupBackground(background);
        return this;
    }


    /**
     * <p>
     * 设置PopupWindow背景View，因为背景全屏的原因，该View将会被强制设为MATCH_PARENT/MATCH_PARENT</strong>
     * </p>
     *
     * @param backgroundView 背景View
     */
    public BasePopupWindow setBackgroundView(View backgroundView) {
        mHelper.setBackgroundView(backgroundView);
        return this;
    }

    /**
     * <p>
     * 获取当前PopupWindow背景
     * </p>
     *
     * @return 背景
     */
    public Drawable getPopupBackground() {
        return mHelper.getPopupBackground();
    }

    /**
     * <p>
     * 设置PopupWindow弹出时是否模糊背景。
     * <br>
     * <br>
     * 在使用模糊背景前，您可以通过{@link #setBlurOption(PopupBlurOption)}传入模糊配置。
     * <br>
     * <br>
     * <strong>本方法默认模糊当前Activity的DecorView</strong>
     * </p>
     *
     * @param blurBackgroundEnable true for blur decorView
     */
    public BasePopupWindow setBlurBackgroundEnable(boolean blurBackgroundEnable) {
        return setBlurBackgroundEnable(blurBackgroundEnable, null);
    }

    /**
     * <p>
     * 设置PopupWindow弹出时是否模糊背景。
     * <br>
     * <br>
     * 在使用模糊背景前，您可以通过{@link #setBlurOption(PopupBlurOption)}传入模糊配置。
     * <br>
     * <br>
     * 本方法允许您传入一个初始化监听，您可以在{@link OnBlurOptionInitListener#onCreateBlurOption(PopupBlurOption)}中进行展示前的最后一次修改
     * </p>
     *
     * @param blurBackgroundEnable true for blur decorView
     * @param optionInitListener   初始化回调
     */
    public BasePopupWindow setBlurBackgroundEnable(boolean blurBackgroundEnable, OnBlurOptionInitListener optionInitListener) {
        Activity activity = getContext();
        if (activity == null) {
            onLogInternal("无法配置默认模糊脚本，因为context不是activity");
            return this;
        }
        PopupBlurOption option = null;
        if (blurBackgroundEnable) {
            option = new PopupBlurOption();
            option.setFullScreen(true)
                    .setBlurInDuration(-1)
                    .setBlurOutDuration(-1);
            if (optionInitListener != null) {
                optionInitListener.onCreateBlurOption(option);
            }
            View decorView = getDecorView();
            if (decorView instanceof ViewGroup && decorView.getId() == android.R.id.content) {
                option.setBlurView(((ViewGroup) activity.getWindow().getDecorView()).getChildAt(0));
                option.setFullScreen(true);
            } else {
                option.setBlurView(decorView);
            }
        }

        return setBlurOption(option);
    }

    /**
     * <p>
     * 应用模糊配置，更多详情请参考{@link #setBlurBackgroundEnable(boolean)}或者{@link #setBlurBackgroundEnable(boolean, OnBlurOptionInitListener)}
     * </p>
     *
     * @param option 模糊配置
     */
    public BasePopupWindow setBlurOption(PopupBlurOption option) {
        mHelper.setToBlur(option);
        return this;
    }

    /**
     * 这个方法用于简化您为View设置OnClickListener事件，多个View将会使用同一个点击事件
     */
    protected void setViewClickListener(View.OnClickListener listener, View... views) {
        for (View view : views) {
            if (view != null && listener != null) {
                view.setOnClickListener(listener);
            }
        }
    }

    //region ------------------------------------------Getter/Setter-----------------------------------------------

    /**
     * PopupWindow是否处于展示状态
     */
    public boolean isShowing() {
        return mPopupWindowProxy == null ? false : mPopupWindowProxy.isShowing();
    }

    public OnDismissListener getOnDismissListener() {
        return mHelper.mOnDismissListener;
    }

    /**
     * 设置PopupWindow显示的监听
     */
    public BasePopupWindow setOnPopupWindowShowListener(OnPopupWindowShowListener onPopupWindowShowListener) {
        mHelper.mOnPopupWindowShowListener = onPopupWindowShowListener;
        return this;
    }

    /**
     * <p>
     * 设置dismiss监听
     * </p>
     *
     * @param onDismissListener 监听器
     */
    public BasePopupWindow setOnDismissListener(OnDismissListener onDismissListener) {
        mHelper.mOnDismissListener = onDismissListener;
        return this;
    }

    public OnBeforeShowCallback getOnBeforeShowCallback() {
        return mHelper.mOnBeforeShowCallback;
    }

    /**
     * <p>
     * 当您设置了{@link OnBeforeShowCallback}监听之后，在您调用｛
     * <ul>
     * <li>{@link #showPopupWindow()}</li>
     * <li>{@link #showPopupWindow(int)}</li>
     * <li>{@link #showPopupWindow(View)}</li>
     * </ul>｝
     * <br>
     * 任意一个方法，在show之前回回调到该监听器。
     * <br>
     * </p>
     *
     * @param mOnBeforeShowCallback
     * @return
     * @see OnBeforeShowCallback#onBeforeShow(View, View, boolean)
     */
    public BasePopupWindow setOnBeforeShowCallback(OnBeforeShowCallback mOnBeforeShowCallback) {
        mHelper.mOnBeforeShowCallback = mOnBeforeShowCallback;
        return this;
    }

    /**
     * <p>
     * 设置展示PopupWindow的动画，详情参考{@link #onCreateShowAnimation()}
     * </p>
     *
     * @param showAnimation 展示动画
     */
    public BasePopupWindow setShowAnimation(Animation showAnimation) {
        mHelper.setShowAnimation(showAnimation);
        return this;
    }

    public Animation getShowAnimation() {
        return mHelper.mShowAnimation;
    }

    /**
     * <p>
     * 设置展示PopupWindow的动画，详情参考{@link #onCreateShowAnimator()}
     * </p>
     *
     * @param showAnimator 展示动画
     */
    public BasePopupWindow setShowAnimator(Animator showAnimator) {
        mHelper.setShowAnimator(showAnimator);
        return this;
    }

    public Animator getShowAnimator() {
        return mHelper.mShowAnimator;
    }

    /**
     * <p>
     * 设置退出PopupWindow的动画，详情参考{@link #onCreateDismissAnimation()}
     * </p>
     *
     * @param dismissAnimation 退出动画
     */
    public BasePopupWindow setDismissAnimation(Animation dismissAnimation) {
        mHelper.setDismissAnimation(dismissAnimation);
        return this;
    }

    public Animation getDismissAnimation() {
        return mHelper.mDismissAnimation;
    }

    /**
     * <p>
     * 设置退出PopupWindow的动画，详情参考{@link #onCreateDismissAnimator()}
     * </p>
     *
     * @param dismissAnimator 退出动画
     */
    public BasePopupWindow setDismissAnimator(Animator dismissAnimator) {
        mHelper.setDismissAnimator(dismissAnimator);
        return this;
    }

    public Animator getDismissAnimator() {
        return mHelper.mDismissAnimator;
    }

    /**
     * <p>
     * 获取context，请留意是否为空{@code null}
     * </p>
     *
     * @return 返回对应的context。如果为空，则返回{@code null}
     */
    public Activity getContext() {
        return mContext;
    }

    /**
     * <p>
     * 获取PopupWindow的根布局
     * </p>
     *
     * @see #onCreateContentView()，该布局在这里初始化。
     */
    public View getContentView() {
        return mContentView;
    }

    /**
     * <p>
     * 获取PopupWindow执行动画的View
     * </p>
     * <br>
     * 如果{@link #onCreateAnimateView()}返回为空，则返回contentView（{@link #onCreateContentView()}）
     */
    public View getDisplayAnimateView() {
        return mDisplayAnimateView;
    }

    /**
     * 获取PopupWindow实例
     *
     * @return
     */
    public PopupWindow getPopupWindow() {
        return mPopupWindowProxy;
    }

    public int getOffsetX() {
        return mHelper.getOffsetX();
    }

    /**
     * 设定x位置的偏移量(中心点在popup的左上角)
     * <p>
     *
     * @param offsetX
     */
    public BasePopupWindow setOffsetX(int offsetX) {
        mHelper.offsetX = offsetX;
        return this;
    }

    public int getOffsetY() {
        return mHelper.getOffsetY();
    }

    /**
     * 设定y位置的偏移量(中心点在popup的左上角)
     *
     * @param offsetY
     */
    public BasePopupWindow setOffsetY(int offsetY) {
        mHelper.offsetY = offsetY;
        return this;
    }

    public int getPopupGravity() {
        return mHelper.getPopupGravity();
    }

    /**
     * <p>
     * 设置参考方向  {@link Gravity}
     * <br>
     * <ul>
     * <li> 不跟anchorView联系的情况下，gravity意味着在整个view中的方位{@link #showPopupWindow()}</li>
     * <li> 如果跟anchorView联系，gravity意味着以anchorView为中心的方位{@link #showPopupWindow(View)}</li>
     * </ul>
     *
     * @param popupGravity
     */
    public BasePopupWindow setPopupGravity(int popupGravity) {
        mHelper.popupGravity = popupGravity;
        return this;
    }

    /**
     * <p>
     * 设置参考方向 {@link Gravity}
     * <br>
     * <ul>
     * <li> 不跟anchorView联系的情况下，gravity意味着在整个view中的方位{@link #showPopupWindow()}</li>
     * <li> 如果跟anchorView联系，gravity意味着以anchorView为中心的方位{@link #showPopupWindow(View)}</li>
     * </ul>
     *
     * @param mode <ul><li>GravityMode.RELATIVE_TO_ANCHOR：该模式将会以Anchor作为参考点，表示Popup处于该Anchor的哪个位置</li>
     *                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 <li>GravityMode.ALIGN_TO_ANCHOR_SIDE：该模式将会以Anchor作为参考点，表示Popup对齐Anchor的哪条边</li>
     *                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 </ul>
     */
    public BasePopupWindow setPopupGravity(GravityMode mode, int popupGravity) {
        mHelper.setPopupGravity(mode, popupGravity);
        return this;
    }

    /**
     * 设置参考模式 {@link Gravity}
     *
     * @param mode <ul><li>GravityMode.RELATIVE_TO_ANCHOR：该模式将会以Anchor作为参考点，表示Popup处于该Anchor的哪个位置</li>
     *             <li>GravityMode.ALIGN_TO_ANCHOR_SIDE：该模式将会以Anchor作为参考点，表示Popup对齐Anchor的哪条边</li>
     *             </ul>
     */
    public BasePopupWindow setPopupGravityMode(GravityMode mode) {
        mHelper.setPopupGravityMode(mode, mode);
        return this;
    }

    /**
     * 分别设置垂直方向与水平方向参考模式 {@link Gravity}
     *
     * @param mode <ul><li>GravityMode.RELATIVE_TO_ANCHOR：该模式将会以Anchor作为参考点，表示Popup处于该Anchor的哪个位置</li>
     *             <li>GravityMode.ALIGN_TO_ANCHOR_SIDE：该模式将会以Anchor作为参考点，表示Popup对齐Anchor的哪条边</li>
     *             </ul>
     */
    public BasePopupWindow setPopupGravityMode(GravityMode horizontalMode, GravityMode verticalMode) {
        mHelper.setPopupGravityMode(horizontalMode, verticalMode);
        return this;
    }

    public boolean isAutoLocatePopup() {
        return mHelper.isAutoLocatePopup();
    }

    /**
     * <p>
     * 是否自动设置PopupWindow位置
     * <br>
     * 在设定了clipToScreen的情况下{@link #setClipToScreen(boolean)}
     * <br>
     * 如果当前屏幕不足以完整显示您的PopupWindow，则PopupWindow会自行布置在其镜像位置。
     * <br>
     * <br>
     * <pre>
     * 比如当前PopupWindow显示在某个View的下方，而屏幕下方不够位置展示完整改PopupWindow，
     * 当本设置为true，PopupWindow将会显示在原来的View的上方以满足完整显示PopupWindow的情况。
     * </pre>
     * <br>
     * <br>
     * <strong>如果您配置了{@link #setOffsetY(int)}，则对应的偏移量也是在其适配后的位置生效</strong>
     * </p>
     *
     * @param isAutoLocatePopup 是否自适配
     */
    public BasePopupWindow setAutoLocatePopup(boolean isAutoLocatePopup) {
        mHelper.setFlag(AUTO_LOCATED, isAutoLocatePopup);
        return this;
    }

    /**
     * <p>
     * 获取PoupWindow的高度。
     * <br>
     * <br>
     * 当PopupWindow没show出来的时候高度会是0，此时则返回pre measure的高度，不一定精准
     * </p>
     *
     * @see #preMeasurePopupView(int, int)
     */
    public int getHeight() {
        if (mContentView == null) return mHelper.getPreMeasureHeight();
        return mContentView.getHeight() <= 0 ? mHelper.getPreMeasureHeight() : mContentView.getHeight();
    }

    /**
     * <p>
     * 获取PoupWindow的宽度。
     * <br>
     * <br>
     * 当popupwindow没show出来的时候高度会是0，此时则返回pre measure的宽度，不一定精准
     * </p>
     *
     * @see #preMeasurePopupView(int, int)
     */
    public int getWidth() {
        if (mContentView == null) return mHelper.getPreMeasureWidth();
        return mContentView.getWidth() <= 0 ? mHelper.getPreMeasureWidth() : mContentView.getWidth();
    }

    /**
     * <p>
     * 是否允许点击PopupWindow外部时触发dismiss
     * </p>
     * <br>
     * dismiss popup when touch outside from popup
     *
     * @param dismissWhenTouchOutside true for allow
     * @deprecated please use {@link #setOutSideDismiss(boolean)} instead
     */
    @Deprecated
    public BasePopupWindow setAllowDismissWhenTouchOutside(boolean dismissWhenTouchOutside) {
        setOutSideDismiss(dismissWhenTouchOutside);
        return this;
    }

    /**
     * <p>
     * 是否允许点击PopupWindow外部时触发dismiss
     * </p>
     * <br>
     * dismiss popup when touch outside from popup
     *
     * @param outSideDismiss true for allow
     */
    public BasePopupWindow setOutSideDismiss(boolean outSideDismiss) {
        mHelper.setFlag(OUT_SIDE_DISMISS, outSideDismiss);
        return this;
    }

    /**
     * <p>
     * 是否允许点击PopupWindow拦截事件。
     * <br>
     * <br>
     * 如果允许拦截事件，则PopupWindow外部无法响应事件。
     * </p>
     *
     * @param touchable <ul>
     *                  <li>ture:PopupWindow拦截事件</li>
     *                  <li>false：不拦截事件</li>
     *                  </ul>
     * @deprecated please use {@link #setOutSideTouchable(boolean)} instead
     */
    @Deprecated
    public BasePopupWindow setAllowInterceptTouchEvent(boolean touchable) {
        setOutSideTouchable(!touchable);
        return this;
    }

    /**
     * <p>
     * 是否允许点击PopupWindow拦截事件。
     * <br>
     * <br>
     * 如果允许拦截事件，则PopupWindow外部无法响应事件。
     * </p>
     *
     * @param touchable <ul>
     *                  <li>ture:外部可点击</li>
     *                  <li>false：外部不可点击</li>
     *                  </ul>
     */
    public BasePopupWindow setOutSideTouchable(boolean touchable) {
        mHelper.setFlag(OUT_SIDE_TOUCHABLE, touchable);
        return this;
    }

    /**
     * 该参数决定popupWindow是否被限制在绘制边界
     * <p>
     * <br>
     * <ul>
     * <li>true：PopupWindow将会被限制边界，其动画不可突破其边界</li>
     * <li>false：PopupWindow将不会被限制绘制边界，其动画可突破其边界</li>
     * </ul>
     *
     * @param clipChildren 默认为true
     */
    public BasePopupWindow setClipChildren(boolean clipChildren) {
        mHelper.setFlag(CLIP_CHILDREN, clipChildren);
        return this;
    }

    public boolean isAllowDismissWhenTouchOutside() {
        return mHelper.isOutSideDismiss();
    }

    /**
     * @deprecated please use {@link #isOutSideTouchable()}
     */
    @Deprecated
    public boolean isAllowInterceptTouchEvent() {
        return !mHelper.isOutSideTouchable();
    }

    /**
     * 外部是否可以点击
     */
    public boolean isOutSideTouchable() {
        return mHelper.isOutSideTouchable();
    }

    /**
     * <p>
     * 设置PopupWindow的背景是否对齐到PopupWindow。
     * <br>
     * <br>
     * 默认情况下，PopupWindow背景都是铺满整个屏幕的。
     * 但在某些情况下您可能在PopupWindow之上不需要展示背景，这时候您可以调用这个方法来强制Background对齐到PopupWindow的顶部。
     * </p>
     *
     * @param isAlignBackground 是否对齐背景
     */
    public BasePopupWindow setAlignBackground(boolean isAlignBackground) {
        mHelper.setAlignBackgound(isAlignBackground);
        return this;
    }

    /**
     * <p>
     * 设置PopupWindow的背景对齐PopupWindow的方式，请传入{@link Gravity}中的值
     * <br>
     *
     * @param gravity 请传入{@link Gravity}中的值，传入{@link Gravity#NO_GRAVITY}则意味着不对齐
     */
    public BasePopupWindow setAlignBackgroundGravity(int gravity) {
        mHelper.setAlignBackgroundGravity(gravity);
        return this;
    }

    /**
     * <p>
     * 允许PopupWindow跟某个anchorView关联，其位置，可视性将会跟anchorView同步</>
     * <br>
     * <b>WARN：非常不建议在anchorView频繁变化的情况下使用背景模糊，这会导致较大的性能消耗。</b>
     */
    public BasePopupWindow linkTo(View anchorView) {
        mHelper.linkTo(anchorView);
        return this;
    }

    public BasePopupWindow setWidth(int width) {
        mHelper.setPopupViewWidth(width);
        return this;
    }

    public BasePopupWindow setHeight(int height) {
        mHelper.setPopupViewHeight(height);
        return this;
    }

    /**
     * 设置BasePopup最大宽度
     */
    public BasePopupWindow setMaxWidth(int maxWidth) {
        mHelper.maxWidth = maxWidth;
        return this;
    }

    /**
     * 设置BasePopup最大高度
     */
    public BasePopupWindow setMaxHeight(int maxHeight) {
        mHelper.maxHeight = maxHeight;
        return this;
    }

    /**
     * 设置BasePopup最小宽度
     */
    public BasePopupWindow setMinWidth(int minWidth) {
        mHelper.minWidth = minWidth;
        return this;
    }

    /**
     * 设置BasePopup最小高度
     */
    public BasePopupWindow setMinHeight(int minHeight) {
        mHelper.minHeight = minHeight;
        return this;
    }

    /**
     * 是否允许BasePopup自动调整大小
     * <p>
     * 在{@link #setClipToScreen(boolean)}为true的情况下，BasePopup会针对剩余空间来调整Popup的大小，因此可能出现实际显示过小的情况
     */
    public BasePopupWindow setFitSize(boolean canResize) {
        mHelper.setFlag(FITSIZE, canResize);
        return this;
    }

    /**
     * 设置背景蒙层显示的动画，如果为空，则蒙层不显示动画，直接弹出
     */
    public BasePopupWindow setMaskViewShowAnimation(Animation animation) {
        mHelper.mMaskViewShowAnimation = animation;
        return this;
    }

    /**
     * 设置背景蒙层退出的动画，如果为空，则蒙层不显示动画，直接消失
     */
    public BasePopupWindow setMaskViewDismissAnimation(Animation animation) {
        mHelper.mMaskViewDismissAnimation = animation;
        return this;
    }

    public BasePopupWindow syncMaskAnimationDuration(boolean sync) {
        mHelper.setFlag(BasePopupFlag.SYNC_MASK_ANIMATION_DURATION, sync);
        return this;
    }

    /**
     * 设置键盘监听回调
     */
    public BasePopupWindow setOnKeyboardChangeListener(KeyboardUtils.OnKeyboardChangeListener listener) {
        mHelper.mUserKeyboardStateChangeListener = listener;
        return this;
    }

    /**
     * 设置key event
     */
    public BasePopupWindow setKeyEventListener(KeyEventListener keyEventListener) {
        mHelper.mKeyEventListener = keyEventListener;
        return this;
    }

    /**
     * basepopup宽度跟随anchorview
     */
    public BasePopupWindow setWidthAsAnchorView(boolean widthAsAnchor) {
        mHelper.setFlag(BasePopupFlag.AS_WIDTH_AS_ANCHOR, widthAsAnchor);
        return this;
    }

    /**
     * basepopup高度跟随anchorView
     */
    public BasePopupWindow setHeightAsAnchorView(boolean heightAsAnchor) {
        mHelper.setFlag(BasePopupFlag.AS_HEIGHT_AS_ANCHOR, heightAsAnchor);
        return this;
    }

    /**
     * 设置layoutdirection
     */
    public BasePopupWindow setLayoutDirection(int layoutDirection) {
        mHelper.layoutDirection = layoutDirection;
        return this;
    }

    //endregion

    //region ------------------------------------------状态控制-----------------------------------------------

    void superDismiss() {
        try {
            mPopupWindowProxy.superDismiss();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            mHelper.onDismiss();
        }
    }

    /**
     * 取消一个PopupWindow，如果有退出动画，PopupWindow的消失将会在动画结束后执行
     */
    public void dismiss() {
        dismiss(true);
    }

    /**
     * 取消一个PopupWindow，如果有退出动画，PopupWindow的消失将会在动画结束后执行
     *
     * @param animateDismiss 传入为true，则执行退出动画后dismiss（如果有的话）
     */
    public void dismiss(boolean animateDismiss) {
        if (Looper.myLooper() != Looper.getMainLooper()) {
            throw new CalledFromWrongThreadException(PopupUtils.getString(R.string.basepopup_error_thread));
        }
        if (!isShowing() || mContentView == null) return;
        mHelper.dismiss(animateDismiss);
    }

    /**
     * 直接消掉PopupWindow而不需要动画
     *
     * @deprecated 请使用 {@link #dismiss(boolean)}
     */
    @Deprecated
    public void dismissWithOutAnimate() {
        dismiss(false);
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    public void onDestroy() {
        isDestroyed = true;
        onLogInternal("onDestroy");
        mHelper.forceDismiss();
        if (mPopupWindowProxy != null) {
            mPopupWindowProxy.clear(true);
        }
        if (mHelper != null) {
            mHelper.clear(true);
        }
        ownerAnchorParent = null;
        mAnchorDecorView = null;
        mPopupWindowProxy = null;
        mDisplayAnimateView = null;
        mContentView = null;
        mContext = null;
    }

    private boolean checkPerformShow(View v) {
        boolean result = true;
        if (mHelper.mOnBeforeShowCallback != null) {
            result = mHelper.mOnBeforeShowCallback.onBeforeShow(mContentView, v,
                                                                mHelper.mShowAnimation != null || mHelper.mShowAnimator != null);
        }
        return result;
    }

    /**
     * 捕捉keyevent
     *
     * @param event
     * @return true意味着你已经处理消耗了事件，后续不再传递
     */
    public boolean onDispatchKeyEvent(KeyEvent event) {
        return false;
    }

    /**
     * 捕捉interceptTouchEvent
     *
     * @param event
     * @return true意味着你已经处理消耗了事件，后续不再传递
     */
    public boolean onInterceptTouchEvent(MotionEvent event) {
        return false;
    }

    /**
     * 捕捉touchevent
     *
     * @param event
     * @return true意味着你已经处理消耗了事件，后续不再传递
     */
    public boolean onTouchEvent(MotionEvent event) {
        return false;
    }

    /**
     * 捕捉返回键事件
     *
     * @return true意味着你已经处理消耗了事件，后续不再传递
     */
    public boolean onBackPressed() {
        if (mHelper.isBackPressEnable()) {
            dismiss();
            return true;
        }
        return false;
    }

    /**
     * PopupWindow外的事件点击回调，请注意您的PopupWindow大小
     *
     * @return true意味着你已经处理消耗了事件，后续不再传递
     */
    public boolean onOutSideTouch() {
        boolean result = false;
        if (mHelper.isOutSideDismiss()) {
            dismiss();
            result = true;
        } else if (!mHelper.isOutSideTouchable()) {
            result = true;
        }
        return result;
    }

    //endregion

    //region ------------------------------------------callback-----------------------------------------------

    /**
     * 在anchorView上方显示，autoLocatePopup为true时适用
     */
    public void onAnchorTop() {

    }

    /**
     * 在anchorView下方显示，autoLocatePopup为true时适用
     */
    public void onAnchorBottom() {

    }

    /**
     * 在anchorView上方显示，autoLocatePopup为true时适用
     *
     * @param mPopupView {@link #onCreateContentView()}返回的View
     * @param anchorView {@link #showPopupWindow(View)}传入的View
     * @see #onAnchorTop()
     * @deprecated 因为contentView和anchorView应由用户自行保存决定，此处不再返回
     */
    @Deprecated
    public void onAnchorTop(View mPopupView, View anchorView) {

    }

    /**
     * 在anchorView下方显示，autoLocatePopup为true时适用
     *
     * @param mPopupView {@link #onCreateContentView()}返回的View
     * @param anchorView {@link #showPopupWindow(View)}传入的View
     * @see #onAnchorBottom()
     * @deprecated 因为contentView和anchorView应由用户自行保存决定，此处不再返回
     */
    @Deprecated
    public void onAnchorBottom(View mPopupView, View anchorView) {

    }

    @Override
    public void onDismiss() {
        if (mHelper.mOnDismissListener != null) {
            mHelper.mOnDismissListener.onDismiss();
        }
        isExitAnimatePlaying = false;
    }

    /**
     * 在PopupWindow显示后回调该方法
     */
    public void onShowing() {

    }

    /**
     * 返回BasePopup布局中的位置信息
     */
    public void onPopupLayout(@NonNull Rect popupRect, @NonNull Rect anchorRect) {

    }

    public int computeGravity(@NonNull Rect popupRect, @NonNull Rect anchorRect) {
        return PopupUiUtils.computeGravity(popupRect, anchorRect);
    }

    //endregion

    //region ------------------------------------------tools-----------------------------------------------

    protected float dipToPx(float dip) {
        if (getContext() == null) return dip;
        return dip * getContext().getResources().getDisplayMetrics().density + 0.5f;
    }

    public static void setDebugMode(boolean debugMode) {
        PopupLog.setOpenLog(debugMode);
    }

    /**
     * 日志输出口
     */
    protected void onLogInternal(String msg) {
        PopupLog.d(TAG, msg);
    }

    private String ownerParentLog() {
        return PopupUtils.getString(R.string.basepopup_host, String.valueOf(ownerAnchorParent));
    }
//endregion

    //region------------------------------------------Interface-----------------------------------------------
    public interface OnBeforeShowCallback {
        /**
         * <p>
         * 在PopupWindow展示出来之前，如果您设置好了该监听器{@link #setOnBeforeShowCallback(OnBeforeShowCallback)}
         * 那么show之前将会回调到本方法，在这里您可以进一步决定是否可以展示PopupWindow
         * </p>
         *
         * @param contentView    PopupWindow的ContentView
         * @param anchorView     锚点View
         * @param hasShowAnimate 是否有showAnimation
         * @return <ul>
         * <li>【true】：允许展示PopupWindow</li>
         * <li>【false】：不允许展示PopupWindow</li>
         * </ul>
         */
        boolean onBeforeShow(View contentView, View anchorView, boolean hasShowAnimate);


    }

    public interface OnBlurOptionInitListener {
        void onCreateBlurOption(PopupBlurOption option);
    }

    public static abstract class OnDismissListener implements PopupWindow.OnDismissListener {
        /**
         * <p>
         * 在PopupWindow消失之前，如果您设置好了该监听器{@link #setOnDismissListener(OnDismissListener)}
         * 那么dismiss之前将会回调到本方法，在这里您可以进一步决定是否可以继续取消PopupWindow
         * </p>
         *
         * @return <ul>
         * <li>【true】：继续取消PopupWindow</li>
         * <li>【false】：不允许取消PopupWindow</li>
         * </ul>
         */
        public boolean onBeforeDismiss() {
            return true;
        }

        /**
         * <p>
         * 在PopupWindow消失之前，如果您设置好了该监听器{@link #setOnDismissListener(OnDismissListener)}
         * 如果有退出动画，则在退出动画播放的时候会回调该方法
         * </p>
         */
        public void onDismissAnimationStart() {
        }
    }

    /**
     * 当PopupWindow显示在Window上回回调给该接口监听（如果设置了监听的话）
     */
    public interface OnPopupWindowShowListener {
        void onShowing();
    }

    /**
     * key事件，如果返回true，则不再回调BasePopupWindow的onDispatchKeyEvent
     *
     * @see #onDispatchKeyEvent(KeyEvent)
     */
    public interface KeyEventListener {
        boolean onKey(KeyEvent keyEvent);
    }
//endregion

    public static final class CalledFromWrongThreadException extends AndroidRuntimeException {
        public CalledFromWrongThreadException(String msg) {
            super(msg);
        }
    }

}
